/**
 * (c) 2017 Shoreditch Ops Ltd. All rights reserved.
 *
 * This file is part of Artillery Pro (https://artillery.io)
 *
 * NOTICE: All information contained herein is, and remains
 * the property of Shoreditch Ops Ltd. The intellectual and
 * technical concepts contained herein are proprietary to
 * Shoreditch Ops Ltd and are protected by copyright law.
 *
 * Modification and/or redistribution of this material is
 * strictly forbidden unless prior written permission is
 * obtained from Shoreditch Ops Ltd.
 *
 */

'use strict';

const fs = require('fs');
const path = require('path');

const traverse = require('traverse');

const chalk = require('chalk');

const tryResolve = require('try-require').resolve;
let pro = null;
if (tryResolve('artillery-pro')) {
  pro = require('artillery-pro');
};

const debug = require('debug')('plugin:fileUploads');

module.exports = {
  Plugin: FileUploadPlugin
};

function FileUploadPlugin(script, events, opts) {
  this._activated = (pro !== null && pro.isActivatedSync());

  if (!this._activated) {
    console.log(
      chalk.yellow(
        'Artillery',
        'Pro',
        'not detected.',
        'Please install',
        'and activate Artillery',
        'Pro.\nThis',
        'plugin must',
        'be used with Artillery',
        'Pro.'
      )
    );
  }

  const basePath = path.dirname(opts.absoluteScriptPath);

  if (!script.config.processor) {
    script.config.processor = {};
  }

  script.config.processor.fileUploadPluginBeforeRequest = function(
    requestParams,
    userContext,
    events,
    done) {

    if (!requestParams || !requestParams.formData) {
      return done();
    }

    traverse(requestParams).forEach(function(o) {
      if (o && o.fromFile) {
        const filename = path.resolve(
          basePath,
          renderVariables(o.fromFile, userContext.vars));
        debug(`Updating`, o, `with`, filename);
        const stream = fs.createReadStream(filename);
        // in-place update:
        this.update(stream);
      }
    });

    return done();
  };

  script.scenarios.forEach(function(scenario) {
    if (!scenario.beforeRequest) {
      scenario.beforeRequest = [];
    }

    scenario.beforeRequest.push('fileUploadPluginBeforeRequest');
  });

  debug('Plugin initialized');
}

function renderVariables (str, vars) {
  const RX = /{{{?[\s$\w]+}}}?/g;
  let rxmatch;
  let result = str.substring(0, str.length);

  while (result.search(RX) > -1) {
    let templateStr = result.match(RX)[0];
    const varName = templateStr.replace(/{/g, '').replace(/}/g, '').trim();

    let varValue = vars[varName] || '';
    if (typeof varValue === 'object') {
      varValue = JSON.stringify(varValue);
    }
    result = result.replace(templateStr, varValue);
  }

  return result;
}
